@@ -51,6 +51,9 @@ const LEVEL_LABEL_WIDTH: usize = 9;
 /// Padding to the left of the log message
 const MESSAGE_LEFT_PADDING: usize = 3;
 
+/// Logfile directory location
+const LOGFILE_DIR: &str = "logs";
+
 #[cfg(test)]
 pub const STDOUT_FILENAME: &str = "logs/stdout_redirect.log";
 #[cfg(test)]
@@ -62,6 +65,7 @@ pub const FILE_OUT_FILENAME: &str = "logs/file_out_redirect.log";
 ///////////////////////////////////////////////////////////////////////////////
 
 pub struct Receiver {
+    logfile_prefix: &'static str,
     logger_rx: mpsc::Receiver<Command>,
     output_level: Level,
     output_stream: OutputStream,
@@ -76,12 +80,14 @@ pub struct Receiver {
 impl Receiver {
     /// Fully-qualified constructor
     pub fn new(
+        logfile_prefix: &'static str,
         logger_rx: mpsc::Receiver<Command>,
         output_level: Level,
         output_stream: OutputStream,
         msg_count: Arc<AtomicU64>,
     ) -> Self {
         Self {
+            logfile_prefix,
             logger_rx,
             output_level,
             output_stream,
@@ -103,14 +109,13 @@ impl Receiver {
         );
 
         // Open a logfile, creating logs directory if necessary
-        let logfile_dir = "logs";
-        //FEAT: Genericize this
         let logfile_name = format!(
-            "mt_log_{}.log",
+            "{}_{}.log",
+            self.logfile_prefix,
             start_time.format(FILE_TIMESTAMP_FORMAT)
         );
 
-        let mut path_buf = PathBuf::from(logfile_dir);
+        let mut path_buf = PathBuf::from(LOGFILE_DIR);
         if !path_buf.as_path().exists() {
             match fs::create_dir(path_buf.as_path()) {
                 Ok(()) => (),
@@ -152,9 +157,7 @@ impl Receiver {
                 // Handle command based on type
                 match logger_cmd {
                     /* Messages */
-                    Command::LogMsg(log_tuple) => {
-                        self.record_msg(&mut logfile, log_tuple)
-                    }
+                    Command::LogMsg(log_tuple) => self.record_msg(&mut logfile, log_tuple),
 
                     /* Configuration Commands */
                     Command::SetOutputLevel(output_level) => {
@@ -177,7 +180,10 @@ impl Receiver {
                                 level: Level::Error,
                                 fn_name: "LOG_RECEIVER_FLUSH_COMMAND".to_string(),
                                 line: line!(),
-                                msg: format!("Encountered SendError '{}' when sending flush ACK message.", e),
+                                msg: format!(
+                                    "Encountered SendError '{}' when sending flush ACK message.",
+                                    e
+                                ),
                             };
 
                             self.record_msg(&mut logfile, err_tuple);
@@ -254,10 +260,14 @@ impl Receiver {
                 );
 
                 //FEAT: Avoid spewing the same error if a file explodes or something
-                logfile.write_all(msg_formatted.as_bytes())
-                    .unwrap_or_else(
-                        |err| eprintln!("{}: Encountered error '{}' while attempting to write to log file.", log_tuple.timestamp, err)
-                    );
+                logfile
+                    .write_all(msg_formatted.as_bytes())
+                    .unwrap_or_else(|err| {
+                        eprintln!(
+                            "{}: Encountered error '{}' while attempting to write to log file.",
+                            log_tuple.timestamp, err
+                        )
+                    });
 
                 #[cfg(test)]
                 {
